#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>

#include <windows.h>

#include <MagnaLoad.h>


typedef enum {
	ARx,
	ALx,
	WRx,
	UnKn
} IntrumentFamily;


int main(int argc, char **argv)
{
	ViStatus status;
	ViSession session;
	ViChar userInputChar[32];
	int controlModeSelector = 0;
	int controlMode;
	ViChar visaResourceStr[256];
	ViChar instrumentModel[256];

	IntrumentFamily instrumentFamily = UnKn;
	ViReal64 currVal = 0;
	ViReal64 voltVal = 0;
	ViReal64 resVal = 0;
	ViReal64 powerVal = 0;


	// Setup VISA resource descriptor - how the driver will connect to the power supply.

	//Examples of other valid VISA resource descriptors
	//"TCPIP::MPE12345678::50505::SOCKET";   //NETBIOS name direct to TCP port
	//"TCPIP::MPE12345678::inst0::INSTR";    //NETBIOS name to LXI (via UDP and TCP)
	//"TCPIP::192.168.1.100::50505::SOCKET"; //IP address direct to TCP port
	//"GPIB0::13::INSTR";                    //GPIB address 13
	//"ASRL1";                               //COM Port 1

	printf("Enter IP address or host name for device: ");
	scanf_s("%s", userInputChar, (unsigned)_countof(userInputChar));
	sprintf_s(visaResourceStr, 256, "TCPIP::%s::inst0::INSTR", userInputChar);

	//Driver Identity properties - Initialize not required
	//
	ViChar buffer[1024];
	status = MagnaLoad_GetAttributeViString(VI_NULL, VI_NULL, MAGNALOAD_ATTR_SPECIFIC_DRIVER_PREFIX, 1024, buffer);
	if (status != VI_SUCCESS)
		exit(1);
	else
		printf("Prefix: %s\n", buffer);
	status = MagnaLoad_GetAttributeViString(VI_NULL, VI_NULL, MAGNALOAD_ATTR_SPECIFIC_DRIVER_REVISION, 1024, buffer);
	if (status != VI_SUCCESS)
		exit(1);
	else
		printf("Revision: %s\n", buffer);
	status = MagnaLoad_GetAttributeViString(VI_NULL, VI_NULL, MAGNALOAD_ATTR_SPECIFIC_DRIVER_VENDOR, 1024, buffer);
	if (status != VI_SUCCESS)
		exit(1);
	else
		printf("Vendor: %s\n", buffer);
	status = MagnaLoad_GetAttributeViString(VI_NULL, VI_NULL, MAGNALOAD_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 1024, buffer);
	if (status != VI_SUCCESS)
		exit(1);
	else
		printf("Description: %s\n", buffer);

	// Initialize the connection to the instrument
	//
	status = MagnaLoad_InitWithOptions(visaResourceStr, VI_TRUE, VI_TRUE,
		                               "QueryInstrStatus=true, Cache=true, RangeCheck=true, Simulate=false, DriverSetup= Model=MagnaLoad, Trace=false", &session);

	if (status != VI_SUCCESS)
		exit(1);

	if (session == VI_NULL)
		exit(1);
	
	// Driver identity (session required)
	//
	status = MagnaLoad_GetAttributeViString(session, VI_NULL, MAGNALOAD_ATTR_INSTRUMENT_MODEL, 256, instrumentModel);
	if(status != VI_SUCCESS)
		exit(1);
	printf("InstrumentModel: %s\n", instrumentModel);
	status = MagnaLoad_GetAttributeViString(session, VI_NULL, MAGNALOAD_ATTR_INSTRUMENT_FIRMWARE_REVISION, 1024, buffer);
	if(status != VI_SUCCESS)
		exit(1);
	printf("InstrumentFirmwareRevision: %s\n", buffer);
	status = MagnaLoad_GetAttributeViString(session, VI_NULL, MAGNALOAD_ATTR_INSTRUMENT_MANUFACTURER, 1024, buffer);
	if (status != VI_SUCCESS)
		exit(1);
	printf("InstrumentManufacturer: %s\n", buffer);

	status = MagnaLoad_ClearError(session);
	if (status != VI_SUCCESS)
		exit(1);

	if (strncmp(instrumentModel, "ALx", 3) == 0)
		instrumentFamily = ALx;
	else if (strncmp(instrumentModel, "ARx", 3) == 0)
		instrumentFamily = ARx;
	else if (strncmp(instrumentModel, "WRx", 3) == 0)
		instrumentFamily = WRx;
	else
		exit(1);

	ViChar userPrompt[512];
	if (instrumentFamily == ALx)
	{
		sprintf_s(userPrompt, 512, "\nControl Modes:\n 1 - Current\n 2 - Voltage\n 3 - Resistance\n 4 - Power\n%sSelect control mode: ", "");
	}
	else
	{
		sprintf_s(userPrompt, 512, "\nControl Modes:\n 1 - Current\n 2 - Voltage\n 3 - Resistance\n 4 - Power\n%sSelect control mode: ", " 5 - Reostat\n");
	}
	printf(userPrompt);
	scanf_s("%i", &controlModeSelector);

	if (instrumentFamily == ALx && (controlModeSelector < 1 || controlModeSelector > 4))
	{
		printf("Control Mode value out of range.\n");
		exit(1);
	}
	else if (instrumentFamily != ALx && (controlModeSelector < 1 || controlModeSelector > 5))
	{
		printf("Control Mode value out of range.\n");
		exit(1);
	}

	switch (controlModeSelector)
	{
	case 1:
		controlMode = MAGNALOAD_VAL_CONTROL_MODE_CURRENT;
		break;
	case 2:
		controlMode = MAGNALOAD_VAL_CONTROL_MODE_VOLTAGE;
		break;
	case 3:
		controlMode = MAGNALOAD_VAL_CONTROL_MODE_RESISTANCE;
		break;
	case 4:
		controlMode = MAGNALOAD_VAL_CONTROL_MODE_POWER;
		break;
	case 5:
		controlMode = MAGNALOAD_VAL_CONTROL_MODE_RHEOSTAT;
		break;
	}

	printf("Enter Set Points\nCurrent [A]: ");
	scanf_s("%lf", &currVal);
	printf("Voltage [V]: ");
	scanf_s("%lf", &voltVal);
	printf("Power [W]: ");
	scanf_s("%lf", &powerVal);
	printf("Resitance [Ohm]: ");
	scanf_s("%lf", &resVal);

	// Set instrument control mode
	status = MagnaLoad_SetAttributeViInt32(session, VI_NULL, MAGNALOAD_ATTR_CONTROL_MODE, controlMode);
	if (status != VI_SUCCESS)
		exit(1);

	// Set instrument set points
	status = MagnaLoad_ConfigureAllSetpoints(session, currVal, voltVal, resVal, powerVal);
	if (status != VI_SUCCESS)
		exit(1);

	// Enable instrument input
	status = MagnaLoad_StartInput(session);
	if (status != VI_SUCCESS)
		exit(1);
	else
		Sleep(500);

	char c;
	printf("\nPress ESC to exit.\n");
	
	for (;;) {
		if (_kbhit()) {
			c = _getch();
			if (c == 27)
				break;
		}

		status = MagnaLoad_MeasureAll(session, &currVal, &voltVal, &resVal, &powerVal);
		if (status != VI_SUCCESS)
			break;
		else
			printf("Current %lf A, Voltage %lf V, Resistance %lf Ohm, Power %lf W\n");
		Sleep(1000);
	}

	// Disable intrument input
	status = MagnaLoad_StopInput(session);

	// Close the connection to the instrument
	//
	status = MagnaLoad_close(session);
	session = VI_NULL;

	return 0;
}