﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Ivi.Driver.Interop;
using MagnaPower.MagnaLoad.Interop;

namespace SimpleControl
{
    public enum InstrumentFamily
    {
        ARx,
        ALx,
        WRx
    }

    class UnknowInstrumentModel : Exception
    {
        public UnknowInstrumentModel(string message)
        {

        }
    }

    public class Program
    {
        [STAThread]
        public static void Main(string[] args)
        {
            String userInput, userPrompt, visaResourceStr;
            int controlModeSelector;
            MagnaLoadControlModeEnum controlMode = MagnaLoadControlModeEnum.MagnaLoadControlModeCurrent;
            InstrumentFamily instrumentFamily;
            IMagnaLoad magnaLoadDriver = null;
            IMagnaLoadInput magnaLoadInput;
            IMagnaLoadSystem magnaLoadSystem;

            double currVal = 0;
            double voltVal = 0;
            double resVal = 0;
            double powerVal = 0;

            // Setup VISA resource descriptor - how the driver will connect to the power supply.

            //Examples of other valid VISA resource descriptors
            //"TCPIP::MPE12345678::50505::SOCKET";   //NETBIOS name direct to TCP port
            //"TCPIP::MPE12345678::inst0::INSTR";    //NETBIOS name to LXI (via UDP and TCP)
            //"TCPIP::192.168.1.100::50505::SOCKET"; //IP address direct to TCP port
            //"GPIB0::13::INSTR";                    //GPIB address 13
            //"ASRL1";                               //COM Port 1


            Console.Write("Enter IP address or host name for device: ");
            userInput = Console.ReadLine();
            visaResourceStr = String.Format("TCPIP::{0}::inst0::INSTR", userInput);

            try
            {
                magnaLoadDriver = new MagnaLoad();

                // IIviDriverIdentity properties - Initialize not required
                //
                Console.WriteLine("========== Driver ===========");
                Console.WriteLine("Identifier: {0}", magnaLoadDriver.Identity.Identifier);
                Console.WriteLine("Revision: {0}", magnaLoadDriver.Identity.Revision);
                Console.WriteLine("Vendor: {0}", magnaLoadDriver.Identity.Vendor);

                // Setup driver-specific initialization options
                var driverOptions = "DriverSetup= Model=MagnaLoad, Trace=false";

                magnaLoadDriver.Initialize(visaResourceStr, true, true, driverOptions);
                // IIviDriverIdentity properties - Initialize required
                //
                Console.WriteLine("========== Instrument =========");
                Console.WriteLine("InstrumentModel: {0}", magnaLoadDriver.Identity.InstrumentModel);
                Console.WriteLine("InstrumentFirmwareRevision: {0}", magnaLoadDriver.Identity.InstrumentFirmwareRevision);
                Console.WriteLine("InstrumentManufacturer: {0}", magnaLoadDriver.Identity.InstrumentManufacturer);

                magnaLoadInput = magnaLoadDriver.Input;
                magnaLoadSystem = magnaLoadDriver as IMagnaLoadSystem;

                magnaLoadDriver.Status.Clear();


                switch (magnaLoadDriver.Identity.InstrumentModel.Substring(0, 3))
                {
                    case "ARx":
                        instrumentFamily = InstrumentFamily.ARx;
                        break;
                    case "ALx":
                        instrumentFamily = InstrumentFamily.ALx;
                        break;
                    case "WRx":
                        instrumentFamily = InstrumentFamily.WRx;
                        break;
                    default:
                        throw new UnknowInstrumentModel("Unrecognized instrument model.");
                }

                userPrompt = "\nControl Modes:\n 1 - Current\n 2 - Voltage\n 3 - Resistance\n 4 - Power\n{0}Select control mode: ";
                if (instrumentFamily == InstrumentFamily.ALx)
                {
                    userPrompt = String.Format(userPrompt, "");
                }
                else
                {
                    userPrompt = String.Format(userPrompt, " 5 - Reostat\n");
                }
                Console.Write(userPrompt);
                userInput = Console.ReadLine();

                if(int.TryParse(userInput, out controlModeSelector))
                {
                    if(instrumentFamily == InstrumentFamily.ALx && (controlModeSelector < 1 || controlModeSelector > 4))
                    {
                        throw new ArgumentException("Control Mode value out of range.");
                    }
                    else if(instrumentFamily != InstrumentFamily.ALx && (controlModeSelector < 1 || controlModeSelector > 5))
                    {
                        throw new ArgumentException("Control Mode value out of range.");
                    }
                }
                else
                {
                    throw new ArgumentException("Invalid Control Mode.");
                }

                switch(controlModeSelector)
                {
                    case 1:
                        controlMode = MagnaLoadControlModeEnum.MagnaLoadControlModeCurrent;
                        break;
                    case 2:
                        controlMode = MagnaLoadControlModeEnum.MagnaLoadControlModeVoltage;
                        break;
                    case 3:
                        controlMode = MagnaLoadControlModeEnum.MagnaLoadControlModeResistance;
                        break;
                    case 4:
                        controlMode = MagnaLoadControlModeEnum.MagnaLoadControlModePower;
                        break;
                    case 5:
                        controlMode = MagnaLoadControlModeEnum.MagnaLoadControlModeRheostat;
                        break;
                }

                Console.WriteLine("\nEnter Set Points");
                Console.Write("Current [A]: ");
                userInput = Console.ReadLine();
                if(double.TryParse(userInput, out currVal))
                {
                    Console.Write("Voltage [V]: ");
                    userInput = Console.ReadLine();
                    if(double.TryParse(userInput, out voltVal))
                    {
                        Console.Write("Power [W]: ");
                        userInput = Console.ReadLine();
                        if(double.TryParse(userInput, out powerVal))
                        {
                            Console.Write("Resistance [Ohm]: ");
                            userInput = Console.ReadLine();
                            if(double.TryParse(userInput, out resVal))
                            {
                                // Set intrument settpoints
                                magnaLoadDriver.ConfigureAllSetpoints(currVal, voltVal, resVal, powerVal);
                            }
                            else
                            {
                                throw new ArgumentException("Invalid Resistance value");
                            }
                        }
                        else
                        {
                            throw new ArgumentException("Invalid Power value");
                        }
                    }
                    else
                    {
                        throw new ArgumentException("Invalid Voltage value");
                    }
                }
                else
                {
                    throw new ArgumentException("Invalid Current value");
                }

                // Set instrument control mode
                magnaLoadSystem.ControlMode = controlMode;

                // enable instrument input
                magnaLoadInput.Start();
                Thread.Sleep(500);

                ConsoleKeyInfo cki = new ConsoleKeyInfo('A', ConsoleKey.A, true, false, false);
                Console.WriteLine("\nPress ESC to stop\n");
                do
                {
                    // read input measurements
                    magnaLoadDriver.MeasureAll(ref currVal, ref voltVal, ref resVal, ref powerVal);
                    Console.WriteLine(String.Format("Current {0}A, Voltage {1}V, Power {2}W, Resistance {3}Ohm", currVal, voltVal, powerVal, resVal));
                    Thread.Sleep(1000);
                    if(Console.KeyAvailable)
                    {
                        cki = Console.ReadKey(true);
                    }

                } while (cki.Key != ConsoleKey.Escape);

                // disable instrument input
                magnaLoadInput.Stop();

                Thread.Sleep(500); 


            }
            catch (Exception e)
            {

                Console.WriteLine(e.Message);
            }
            finally
            {
                if (magnaLoadDriver != null && magnaLoadDriver.Initialized)
                {
                    magnaLoadDriver.Close();
                    magnaLoadDriver = null;
                }
            }
        }
    }
}
